unit uDelfi;

interface

uses Sysutils, Windows, Classes, uBrdList, uBook;

type

TDelfi = class
 private
  TimeGame,TimeInc,TimeMovesUCI,TimeMovesWB,VResign,ResignCount: integer;
  Book: TChessBook;
  SetTimeTick: cardinal;
 public
  bl: TChessBoardList;
  Elo,WBIniHash: integer;
  WBPonderString,Sc1,Sc2: string;
  BlackToMove,ForcedMode,PostMode: boolean;
  WBMode,WBAnalyzing,WBPonder: boolean;
  constructor Create;
  destructor Destroy; override;
  function LoadIni: boolean;
  procedure ClearHash;
  procedure SetHash(mb:integer);
  procedure SetElo(v:integer);
  procedure SetResign(v: integer);
  function PlayMove(s:string): boolean;
  function PlayMoveList(s:string): boolean;
  procedure TakeBack;
  function GoThink: integer;
  procedure GoPonder;
  procedure GoAnalyze;
  function GoBook: boolean;
  function CheckStatus: string;
  function ShouldResign(v:integer): string;
  function SetPosition(sp,sm: string): boolean;
  procedure SetTimeUCI(wtime,btime,winc,binc,movestogo: integer);
  procedure SetTimeWB(mytime,minc,movesgame: integer);
  function CalculateThinkTime: integer;
  function LegalMoves: integer;
end;

TWriteFunction = procedure(s:string);


implementation


uses uTable, uMisc, uGenMove;

{ TDelfi }

constructor TDelfi.Create;
begin
  bl := TChessBoardList.Create;
  bl.WBAbsoluteScore := true;
  Book := TChessBook.Create;
  Book.LoadBook;
  TimeGame := 30*1000;
  TimeInc := 0;
  ForcedMode := true;
  PostMode := true;
  WBMode := true;
  WBPonder := false;
  WBAnalyzing := false;
  WBPonderString := '';
  WBIniHash := 16;
  VResign := 0;
  Elo := 0;
end;

destructor TDelfi.Destroy;
begin
  Book.Free;
  bl.Free;
  inherited;
end;

function TDelfi.LoadIni: boolean;
var
 sl: TStringList;
 s: string;
begin
  result := true;
  s := ExtractFilePath(ParamStr(0)) + 'Delfi.ini';
  sl := TStringList.Create;
  try
   sl.LoadFromFile(s);
  except
   result := false;
   sl.Free;
   exit;
  end;
  Sc1 := '';
  Sc2 := '';
  SetElo(StrToIntDef(sl.Values['ELO_STRENGTH'],0));
  SetResign(StrToIntDef(sl.Values['RESIGN'],7));
  bl.WBAbsoluteScore := trim(sl.Values['ABSOLUTE_SCORE']) <> '0';
  WBIniHash := StrToIntDef(sl.Values['HASH_TABLE'],16);
  sl.Free;
end;

function TDelfi.CheckStatus: string;
var n: integer;
begin
  result := '';
  n := LegalMoves;
  if n = -2 then begin
     result := '1/2-1/2 {Stalemate}';
     exit;
  end;
  if n = -1 then begin
     if BlackToMove
        then result := '1-0 {White mates}'
        else result := '0-1 {Black mates}';
     exit;
  end;
  if bl.IsRepetition(true) then begin
     result := '1/2-1/2 {Triple repetition}';
     exit;
  end;
  with bl.PCurBrd^ do n := AMatStage + BMatStage;
  if (n = 0) or (n = VKNIGHT) or (n = VBISHOP) then begin
     result := '1/2-1/2 {Insufficient material}';
     exit;
  end;
  if bl.PCurBrd.DrawingMoves > 100 then begin
     result := '1/2-1/2 {50 Moves rule}';
     exit;
  end;
end;

function TDelfi.PlayMove(s: string): boolean;
var m: integer;
begin
  m := StringToMove(trim(s));
  result := CheckMoveX(bl.PCurBrd,m,BlackToMove);
  if not result then exit;
  bl.PlayMove(m);
  BlackToMove := not BlackToMove;
end;

procedure TDelfi.TakeBack;
begin
  if bl.Idx = 0 then exit;
  bl.TakeBack;
  BlackToMove := not BlackToMove;
end;

function TDelfi.LegalMoves: integer;
begin
  result := LegalMovesX(bl.PCurBrd,BlackToMove);
end;

function TDelfi.PlayMoveList(s: string): boolean;
var
 v: integer;
 sm: string;
begin
  result := true;
  s := trim(s) + ' ';
  repeat
    v := pos(' ',s);
    sm := trim(copy(s,1,v));
    if sm <> '' then result := PlayMove(sm);
    delete(s,1,v);
    if not result then exit;
  until length(s) < 4;
end;

procedure TDelfi.ClearHash;
begin
  HashTable.Clear(true);
end;

function TDelfi.SetPosition(sp,sm: string): boolean;
var wmove: boolean;
begin
  result := bl.SetPosition(sp,wmove);
  BlackToMove := not wmove;
  if sm <> '' then result := PlayMoveList(sm);
end;

procedure TDelfi.SetTimeUCI(wtime,btime,winc,binc,movestogo: integer);
begin
  SetTimeTick := GetTickCount;
  TimeMovesWB := -1;
  TimeMovesUCI := movestogo;
  if BlackToMove then begin
     TimeGame := btime - 1000;
     TimeInc := binc;
  end
  else begin
     TimeGame := wtime - 1000;
     TimeInc := winc;
  end;
  if TimeGame < 0 then TimeGame := 0;
end;

procedure TDelfi.SetTimeWB(mytime,minc,movesgame: integer);
begin
  SetTimeTick := GetTickCount;
  TimeMovesUCI := -1;
  TimeGame := mytime - 1000;
  if TimeGame < 0 then TimeGame := 0;
  if minc >= 0 then TimeInc := minc;
  if movesgame = 0 then TimeMovesWB := -1;
  if movesgame > 0 then TimeMovesWB := movesgame;
end;

function TDelfi.CalculateThinkTime: integer;
var
 tg,tm,n,t,dt: integer;
const
 TIMEHOUR = 3600*1000;
begin
  dt := integer(GetTickCount - SetTimeTick);
  if (dt < 0) or (dt > TIMEHOUR) then dt := 0;
  tg := TimeGame - dt;
  if tg < 0 then tg := 0;
  tm := TimeMovesUCI;
  n := bl.Idx div 2;
  if TimeMovesWB > 0 then tm := TimeMovesWB - (n mod TimeMovesWB);
  if tm > 0 then begin
     result := tg div (tm + 1);
     exit;
  end;
  n := 61 - n;
  if (n < 36) or (tg < 15000) then n := 36;
  t := tg div n + round(0.91 * TimeInc);
  if t > tg div 3 then t := tg div 3;
  result := t;
end;

function TDelfi.GoThink: integer;
begin
  result := 0;
  if GoBook then exit;
  result := bl.Search(CalculateThinkTime,TimeGame,0,Elo,BlackToMove,false);
end;

procedure TDelfi.GoPonder;
begin
  bl.Search(CalculateThinkTime,TimeGame,0,Elo,BlackToMove,true);
end;

procedure TDelfi.GoAnalyze;
begin
  bl.Search(-1,-1,0,MAXELO,BlackToMove,false);
end;

function TDelfi.GoBook: boolean;
var m: integer;
begin
  m := Book.GetBookMove(bl.PCurBrd,bl.Idx,Elo,BlackToMove);
  result := m <> 0;
  if not result then exit;
  bl.PV.SetPVOne(m);
end;

procedure TDelfi.SetHash(mb: integer);
begin
  HashTable.Alloc(mb);
end;

procedure TDelfi.SetElo(v: integer);
begin
  if (v > 2300) or (v < 1000) then v := MAXELO;
  Elo := v;
end;

procedure TDelfi.SetResign(v: integer);
begin
  VResign := v;
  if VResign < 0 then VResign := 0;
  if (VResign > 0) and (VResign < 3) then VResign := 3;
  VResign := VResign * 100;
end;

function TDelfi.ShouldResign(v:integer): string;
begin
  result := '';
  if (VResign = 0) or (v > -VResign) or (CheckStatus <> '') or
     ((Elo >= 1000) and (Elo < 1300)) then begin
     ResignCount := 0;
     exit;
  end;
  inc(ResignCount);
  if ResignCount < 3 then exit;
  if BlackToMove
     then result := '1-0 {Black resigns}'
     else result := '0-1 {White resigns}';
  ResignCount := 0;
end;

end.
